function Init()
    indicator:name("USD Index");
    indicator:description("Calculates the USD index in on the base of EUR/USD, USD/JPY, GBP/USD, USD/CAD and USD/CHF instruments. You must be subscribed for all these instruments!!!");
    indicator:requiredSource(core.Bar);
    indicator:type(core.Oscillator);

    indicator.parameters:addGroup("Display");
	indicator.parameters:addString("BL", "BAR/LINE", "" , "LINE");
    indicator.parameters:addStringAlternative("BL", "LINE", "" , "LINE");
	indicator.parameters:addStringAlternative("BL", "BAR", "" , "BAR");	
    indicator.parameters:addColor("clrIndex", "Index Color", "", core.rgb(255, 255, 128));
end

local source;
local barSize;
--local usdx;
local close
local open;
local high;
local close;
local loading = false;
local offset;
local weekoffset;
local BL;

-- indexes for the instruments in the data table
local eurusd = 1;
local usdjpy = 2;
local gbpusd = 3;
local usdcad = 4;
local usdchf = 5;
local usdsek = 6;
local first = nil;
local last = nil;
-- data table
local data = {};

-- add a new item into the data table
-- index        - is the index of the item in the table
-- instrument   - the instrument name
-- weight       - the weigth of the instrument
function AddCollectionItem(index, instrument, weight)
    local t, coll, from, to, tmp;
    t = {};
    t.instrument = instrument;
    t.data = nil;
    t.loading = false;
    t.weight = weight;
    t.rqfrom = nil;
    t.rqto = nil;
    data[index] = t;

    if first == nil or first > index then
        first = index;
    end
    if last == nil or last < index then
        last = index;
    end
end

-- initialize the collection of the instruments
function InitCollection()
    -- sum of absolute values of the weights must be 1. negative sign is for the
    -- instrument which has USD as counter currency.
    AddCollectionItem(eurusd, "EUR/USD", -0.576);
    AddCollectionItem(usdjpy, "USD/JPY", 0.136);
    AddCollectionItem(gbpusd, "GBP/USD", -0.119);
    AddCollectionItem(usdcad, "USD/CAD", 0.091);
    AddCollectionItem(usdsek, "USD/SEK", 0.042);
    AddCollectionItem(usdchf, "USD/CHF", 0.036);
end

-- prepare the indicator
function Prepare()
     BL = instance.parameters.BL;  
    source = instance.source;
    host = core.host;
    barSize = source:barSize();
    offset = host:execute("getTradingDayOffset");
    weekoffset = host:execute("getTradingWeekOffset");

    InitCollection();

    local name = profile:id();
    instance:name(name);
    --usdx = instance:addStream("X", core.Line, name .. ".X", "X", instance.parameters.clrIndex, 0);
	
	
	
	if BL == "BAR" then
	close = instance:addStream("close", core.Line, "", "", core.rgb(128, 128, 128), 0);	
	open = instance:addStream("open", core.Line, "", "", core.rgb(128, 128, 128), 0);
    high = instance:addStream("high", core.Line, "", "", core.rgb(128, 128, 128), 0);
    low = instance:addStream("low", core.Line, "", "", core.rgb(128, 128, 128), 0);   
    instance:createCandleGroup("OHLC", "OHLC", open, high, low, close);	
	else
	close = instance:addStream("USDX", core.Line, "USDX", "", core.rgb(128, 128, 128), 0);	
	end
	
	
end

local popen = {};
local pclose = {};
local phigh = {};
local plow = {};
local w = {};

-- get the price of the specified instrument
function GetPrice(index, date)
    local t;

    local from, to, tmp;

    t = data[index];

    assert(t ~= nil, "internal error!");

    if t.data == nil then
        -- data is not loaded yet at all
        if source:isAlive() then
            to = 0;
        else
            to = source:date(source:size() - 1);
        end
        from = source:date(source:first());
        t.data = host:execute("getHistory", index, t.instrument, barSize, from, to, source:isBid());
        t.rqfrom = from;
        t.rqto = to;
        t.loading = true;
        loading = true;
        return 0, 0, 0, 0, 0;
    elseif date < t.rqfrom then
        -- requested date is before the first item of the collection
        -- we have ever requested
        from = date;
        to = t.data:date(0);
        host:execute("extendHistory", index, t.data, from, to);
        t.rqfrom = from;
        t.loading = true;
        loading = true;
        return 0, 0, 0, 0, 0;
    elseif not(source:isAlive()) and date > t.rqto then
        -- requested date is after the last item of the collection
        -- we have ever requested
        to = date;
        from = t.data:date(t.data:size() - 1);
        host:execute("extendHistory", index, t.data, from, to);
        t.rqto = to;
        t.loading = true;
        loading = true;
        return 0, 0, 0, 0, 0;
    end

    local p;
    p = findDateFast(t.data, date, false);
    if p < 0 then
        return 0, 0, 0, 0, 0;
    end
    return t.data.open[p],t.data.high[p],t.data.low[p],t.data.close[p], t.weight;
end

local lastdate = nil;

-- the function which is called to calculate the period
function Update(period, mode)

    if loading or period <= source:first() then
        return ;
    end

    -- do not calculate for the floating candle
    period = period - 1;

    if lastdate ~= nil and source:date(period) == lastdate then
        return ;
    end

    lastdate = source:date(period);

   
	local i, absent, o,h,l,c, b;
	local  x={};
    absent = false;
    for i = first, last, 1 do
      
		 o, h, l, c, b = GetPrice(i, lastdate);
		
        if c == 0 then
            absent = true;
        end
		pclose[i] = c;
        popen[i] = o;
		phigh[i] = h;
		plow[i] = l;		
        w[i] = b;
    end

    if loading then
        close:setBookmark(1, period);
		return ;
    end

    if absent then
        if close:hasData(period - 1) then
			if BL == "BAR" then
			 close[period] = close[period - 1];
			 open[period] = open[period - 1];
			 high[period] = high[period - 1];
			 low[period] = low[period - 1]; 
			 else
			 close[period] = close[period - 1];
			 end
        end
    else
        x[1] = 50.14348112;  x[2] = 50.14348112;  x[3] = 50.14348112;  x[4] = 50.14348112;
        for i = first, last, 1 do
		    if BL == "BAR" then
			    x[4] = x[4] * math.pow(pclose[i], w[i]);
				x[1] = x[1] * math.pow(popen[i], w[i]);
				x[2] = x[2] * math.pow(phigh[i], w[i]);
				x[3] = x[3] * math.pow(plow[i], w[i]);
			else
				x[4] = x[4] * math.pow(pclose[i], w[i]);
			end	
        end
		
		  
		   if BL == "BAR" then
		    close[period] = x[4];
          open[period] = x[1];
		  high[period] = x[2];
		  low[period] = x[3];		 
		  else
		  close[period] = x[4];
		  end
    end

    period = period + 1;

    if period > 0 and period == source:size() - 1 then
       if BL == "BAR" then
			open[period] = close[period - 1];
			 high[period] = close[period - 1];
			low[period] = close[period - 1];		 
			close[period] = close[period - 1];
			 else
			 close[period] = close[period - 1];
			 end
    end
end

function AsyncOperationFinished(cookie)

    local t;
    t = data[cookie];
    t.loading = false;
    for i = first, last, 1 do
        if data[i].loading then
            return ;
        end
    end
    loading = false;

    local period;
    period = close:getBookmark(1);

    if (period < 0) then
        period = 0;
    end
    instance:updateFrom(period);
end


function findDateFast(stream, date, precise)
    local datesec = nil;
    local periodsec = nil;
    local min, max, mid;

    datesec = math.floor(date * 86400 + 0.5)

    min = 0;
    max = stream:size() - 1;

    while true do
        mid = math.floor((min + max) / 2);
        periodsec = math.floor(stream:date(mid) * 86400 + 0.5);
        if datesec == periodsec then
            return mid;
        elseif datesec > periodsec then
            min = mid + 1;
        else
            max = mid - 1;
        end
        if min > max then
            if precise then
                return -1;
            else
                return min - 1;
            end
        end
    end
end

